@@ -4,16 +4,20 @@ package Business::Stripe::WebCheckout;
 #
 # TODO - Post release
 #
-# Improve obtaining success/cancel URLs from environment
-# Add P&P
+# 12-04-21 - Improve obtaining success/cancel URLs from environment
+# 14-04-21 - Add P&P
+# 15-04-21 - Remove extraneous tab characters
+# 15-04-21 - Replicate & cure CPAN Tester bug - line 181
+# 
 
 use HTTP::Tiny;
 use JSON::PP;
+use Data::Dumper;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.1_1';
+our $VERSION = '0.1_2';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -29,7 +33,10 @@ sub new {
 
     $attrs{'cancel-url'}  //= "$ENV{'REQUEST_SCHEME'}://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}";
     $attrs{'success-url'} //= "$ENV{'REQUEST_SCHEME'}://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}";
-	$attrs{'error'}         = 'cancel-url and success-url cannot be derived from the environment and need to be provided' unless ($attrs{'cancel-url'} and $attrs{'success-url'});
+    $attrs{'error'}         = 'cancel-url and success-url cannot be derived from the environment and need to be provided' unless ($attrs{'cancel-url'} and $attrs{'success-url'});
+    
+    # This is changed during testing only
+    $attrs{'url'}		  //= 'https://api.stripe.com/v1/checkout/sessions';
 
     $attrs{'error'} = 'Public API key provided is not a valid key' if $attrs{'api-public'} and $attrs{'api-public'} !~ /^pk_/;
     $attrs{'error'} = 'Secret API key provided is not a valid key' unless $attrs{'api-secret'} =~ /^sk_/;
@@ -147,38 +154,57 @@ sub _create_intent {
 	my $headers = {
 		'Authorization' => 'Bearer ' . $self->{'api-secret'},
 	};
+	
+	# Update URL and headers during stripe-live tests
+	if ($self->{'url'} =~ /^https:\/\/www\.boddison\.com/) {
+		$headers->{'BODTEST'} = __PACKAGE__ . " v$VERSION";
+		$headers->{'Authorization'} = undef,
+		$self->{'url'} .= '?fail' if $self->{'api-test-fail'};
+	}
+	
 	my $vars = {
 		'headers'		=> $headers,
+		'agent'			=> 'Perl-WebCheckout/$VERSION'
 	};
 	my $payload = {
 	    'cancel_url'    			=> $self->{'cancel-url'},
 	    'success_url'   			=> $self->{'success-url'},
 	    'payment_method_types[0]'	=> 'card',
 	    'mode'						=> 'payment',
-	    'metadata[testing]'			=> 500,
-	    'shipping_rates[0]'			=> 'shr_1Ifuh6An8dneEIsYRKWMZ5z8',
-	    'shipping_address_collection[allowed_countries][0]'	=> 'GB',
 	    'client_reference_id'		=> $self->{'reference'},
 	};
 	$payload->{'customer_email'} = $self->{'email'} if $self->{'email'};
 	
 	my $i = 0;
 	foreach my $prod(@{$self->{'trolley'}}) {
-		$payload->{"line_items[$i][currency]"}		= $self->{'currency'};
-		$payload->{"line_items[$i][name]"}			= $prod->{'name'};
-		$payload->{"line_items[$i][description]"}	= $prod->{'description'} if $prod->{'description'};
-		$payload->{"line_items[$i][quantity]"}		= $prod->{'qty'};
-		$payload->{"line_items[$i][amount]"}		= $prod->{'price'};
+		$payload->{"line_items[$i][currency]"}      = $self->{'currency'};
+		$payload->{"line_items[$i][name]"}          = $prod->{'name'};
+		$payload->{"line_items[$i][description]"}   = $prod->{'description'} if $prod->{'description'};
+		$payload->{"line_items[$i][quantity]"}      = $prod->{'qty'};
+		$payload->{"line_items[$i][amount]"}        = $prod->{'price'};
 		$i++;
 	}
 
-	my $response = $http->post_form('https://api.stripe.com/v1/checkout/sessions', $payload, $vars);
+	my $response = $http->post_form($self->{'url'}, $payload, $vars);
 	
 	$self->{'error'} = '';
 	if ($response->{'success'}) {
 		$self->{'intent'} = $response->{'content'};
 	} else {
-		$self->{'error'}  = decode_json($response->{'content'})->{'error'}->{'message'};
+#		open my $fh, '>', "C:\\WebCheckout.debug";
+#		print $fh $self->{'url'} . "\n\n";
+#		print $fh $response->{'content'} . "\n\n";
+#		print $fh $response->{'status'};
+#		close $fh;
+		my $content = $response->{'content'};
+		eval {
+			$content = decode_json($response->{'content'});
+		};
+		if ($@) {
+			$self->{'error'} = $content;
+		} else {
+			$self->{'error'} = $content->{'error'}->{'message'};
+		}
 	}
 }
 
@@ -341,7 +367,7 @@ Next, products are assembled in the Trolley.  There are methods to add, update,
   );
   my @products = $stripe->list_products;
 
-Once the Trolley contains all the products, the user is redirected to the Stripe hosted checkout where they pay for the Trolley.  Once this happens, Stripe returns to your site using one of the URLs provided delending on whether the payment was successful or not.  Where no return URLs are provide, the script URL is used although in practice this is not usually sufficient and return URLs will be needed.
+Once the Trolley contains all the products, the user is redirected to the Stripe hosted checkout where they pay for the Trolley.  Once this happens, Stripe returns to your site using one of the URLs provided depending on whether the payment was successful or not.  Where no return URLs are provided, the script URL is used although in practice this is not usually sufficient and return URLs will be needed.
 
   $stripe->checkout;
   
@@ -409,7 +435,7 @@ Returns a Business::Stripe::WebCheckout even if creation has failed.  The B<succ
 
 =head2 success
 
-Returns true is the last method call was successful
+Returns true if the last method call was successful
 
 =head2 error
 
@@ -477,7 +503,7 @@ On success, returns a hash with the product details.  Each key of the hash corre
 
 =head3 parameters
 
-The following methods all take the following optional parameters.  See B<new> for their descriptions.
+The C<get_intent>, C<get_intent_id>, C<get_ids> and C<checkout> methods all take the following optional parameters.  See C<new> for their descriptions.
 
 =over 4
 
@@ -509,7 +535,7 @@ In addition to the parameters listed above, this method also accepts the followi
 
 =item *
 
-C<public-api> - See B<new>
+C<public-api> - See C<new>
 
 =item *
 
@@ -517,7 +543,7 @@ C<format> - The format of the returned information.  Current options are JSON or
 
 =back
 
-Provides the Public Key and Intent Session ID as these are the two pieces of information required by the Javascript provided by Stripe and te Javacsript provided here.  If text output is used (the default) the Public Key and Intent Session ID are provided as a colon separated string.
+Provides the Public Key and Intent Session ID as these are the two pieces of information required by the Javascript provided by Stripe and the Javacsript provided here.  If text output is used (the default) the Public Key and Intent Session ID are provided as a colon separated string.
 
 =head2 checkout
 
@@ -695,7 +721,7 @@ See L<https://stripe.com/docs/payments/accept-a-payment?integration=checkout>
      # handle errors...
   }
 
-This last example prints out a fully formed HTML to the browser complete with Content-Type header.  If other headers are required, such as Set-Cookie headers, they can be included immediately before printing calling B<checkout>.
+This last example prints out a fully formed HTML document to the browser containing only the C<head> section.  The HTML contains the Javascripe necessary to pass the use to the Stripe hosted checkout.  The HTML is complete with Content-Type header.  If other headers are required, such as Set-Cookie headers, they can be included immediately before calling C<checkout>.
 
   print "Set-cookie: MyCookie=$order_number; path=/\n";
   $stripe->checkout;
